@@ -2,11 +2,13 @@ from typing import Union
 
 import simplejson as json
 
+from worker import constants
 from worker.data.api import API
+from worker.data.enums import RerunMode
 from worker.data.json_encoder import JsonEncoder
-from worker.framework import constants
 from worker.mixins.logging import LoggingMixin
 from worker.mixins.rollbar import RollbarMixin
+from worker.partial_rerun_merge.models import RerunMergeCacheUpdater
 from worker.state.mixins import RedisMixin
 from worker.state.state import State
 
@@ -15,18 +17,17 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
     """
     This is an abstract base module that needs to be extended by an actual module.
     """
+
     # module_key is used for redis access and state of this module
+    app_key = constants.get("global.app-key")
+    app_name = constants.get("global.app-name")
     module_key = "module"
     collection = "collection"
-    module_state_fields = {
-        'last_processed_timestamp': int
-    }
+    module_state_fields = {"last_processed_timestamp": int}
 
     enabled = True
 
     def __init__(self, global_state, *args, **kwargs):
-        self.app_key = constants.get('global.app-key')
-        self.app_name = constants.get('global.app-name')
         self.asset_id = global_state.get("asset_id")
 
         self.global_state = global_state
@@ -77,25 +78,29 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
 
         worker = API()
         last_document = worker.get(
-            path="/v1/data/corva", query=query, collection=self.collection, asset_id=asset_id,
-            sort="{timestamp: -1}", limit=1,
+            path="/v1/data/corva",
+            query=query,
+            collection=self.collection,
+            asset_id=asset_id,
+            sort="{timestamp: -1}",
+            limit=1,
         ).data
 
         if not last_document:
             return 0
 
         last_document = last_document[0]
-        last_processed_timestamp = last_document.get('timestamp', 0)
+        last_processed_timestamp = last_document.get("timestamp", 0)
 
         return last_processed_timestamp
 
     @staticmethod
-    def gather_first_wits_timestamp_since(asset_id: int, since: int, activity_fields=None, operator='eq') -> int:
+    def gather_first_wits_timestamp_since(asset_id: int, since: int, activity_fields=None, operator="eq") -> int:
         """
         Query the Wits collection for this asset_id where state in wits_states and timestamp >= since
         """
 
-        query = '{timestamp#%s#%s}' % ('gt', since)
+        query = "{timestamp#%s#%s}" % ("gt", since)
 
         operator = operator.lower()
 
@@ -113,18 +118,18 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
             else:
                 activity_fields = "'{0}'".format(activity_fields)
 
-            query += 'AND{data.state#%s#%s}' % (operator, activity_fields)
+            query += "AND{data.state#%s#%s}" % (operator, activity_fields)
 
         worker = API()
         first_wits_since = worker.get(
-            path="/v1/data/corva", collection='wits', asset_id=asset_id, sort="{timestamp: 1}", limit=1, query=query
+            path="/v1/data/corva", collection="wits", asset_id=asset_id, sort="{timestamp: 1}", limit=1, query=query
         ).data
 
         if not first_wits_since:
             return 0
 
         first_wits_since = first_wits_since[0]
-        first_wits_since_timestamp = first_wits_since.get('timestamp', 0)
+        first_wits_since_timestamp = first_wits_since.get("timestamp", 0)
 
         return first_wits_since_timestamp
 
@@ -139,7 +144,7 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
         """
         maximum_timestamp = start
         for data in event:
-            if data.get("collection") == "wits" and data.get('data', {}).get('state', None) in activity_fields:
+            if data.get("collection") == "wits" and data.get("data", {}).get("state", None) in activity_fields:
                 maximum_timestamp = max(data.get("timestamp", 0), maximum_timestamp)
 
         return maximum_timestamp
@@ -163,8 +168,12 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
 
         worker = API()
         dataset = worker.get(
-            path="/v1/data/corva", collection=self.collection, asset_id=asset_id, query=query,
-            sort="{timestamp: 1}", limit=limit,
+            path="/v1/data/corva",
+            collection=self.collection,
+            asset_id=asset_id,
+            query=query,
+            sort="{timestamp: 1}",
+            limit=limit,
         ).data
 
         if not dataset:
@@ -197,13 +206,13 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
         :return:
         """
         output = {
-            'timestamp': int(wits.get('timestamp')),
-            'company_id': int(wits.get('company_id')),
-            'asset_id': int(wits.get('asset_id')),
-            'provider': str(wits.get('provider', 'corva')),
-            'version': 1,
-            'collection': self.collection,
-            'data': {}
+            "timestamp": int(wits.get("timestamp")),
+            "company_id": int(wits.get("company_id")),
+            "asset_id": int(wits.get("asset_id")),
+            "provider": str(wits.get("provider", "corva")),
+            "version": 1,
+            "collection": self.collection,
+            "data": {},
         }
 
         enable_output_type_field = constants.get(
@@ -216,7 +225,7 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
         # If a specific module enforces type field in the output collection,
         # build_empty_output adds an additional type field to the output.
         # This can be enforced by setting "enable-output-type-field" to True in the module app constants.
-        output.update({'type': self.module_key})
+        output.update({"type": self.module_key})
 
         return output
 
@@ -224,3 +233,33 @@ class Module(RedisMixin, LoggingMixin, RollbarMixin):
     def format_output(output):
         output = json.dumps(output, cls=JsonEncoder, ignore_nan=True)
         return output
+
+    @classmethod
+    def update_cache(cls, merger: "PartialRerunMerge") -> None:
+        """
+        This function updates the cache of the original asset with the rerun asset's cache
+
+        If in any app that is extending this class, it needs to be overridden, it can be done so.
+
+        :param merger: PartialRerunMerge object
+        :return: None
+        """
+        if merger.rerun_mode == RerunMode.HISTORICAL:
+            return
+
+        original_asset_id = merger.original_asset_id
+        rerun_asset_id = merger.rerun_asset_id
+
+        redis_handler = RedisMixin()
+        redis_handler.asset_id = original_asset_id
+
+        original_state_key = redis_handler.get_formatted_state_key(original_asset_id, cls.app_key, cls.module_key)
+        rerun_state_key = redis_handler.get_formatted_state_key(rerun_asset_id, cls.app_key, cls.module_key)
+
+        original_state = redis_handler.load_state(original_state_key)
+        rerun_state = redis_handler.load_state(rerun_state_key)
+
+        original_state = RerunMergeCacheUpdater.default_updater(original_state, rerun_state)
+
+        redis_handler.state = original_state
+        redis_handler.save_state(original_state_key)
